#ifndef DCF77_h
#define DCF77_h

#if ARDUINO >= 100
#include <Arduino.h> 
#else
#include <WProgram.h> 
#endif

void dcf_init();
void dcf_stop();

enum DCFStatus {
    offline,
    online,
    confirmed
};


class DCF77 {
private:

    //Private variables
    bool initialized;

    // DCF time format structure
    union DCF77Buffer {
        struct {
            //unsigned long long prefix       :21;
            unsigned long long prefix : 17;
            unsigned long long CEST : 1; // CEST 
            unsigned long long CET : 1; // CET 
            unsigned long long unused : 2; // unused bits
            unsigned long long Min : 7; // minutes
            unsigned long long P1 : 1; // parity minutes
            unsigned long long Hour : 6; // hours
            unsigned long long P2 : 1; // parity hours
            unsigned long long Day : 6; // day
            unsigned long long Weekday : 3; // day of week
            unsigned long long Month : 5; // month
            unsigned long long Year : 8; // year (5 -> 2005)
            unsigned long long P3 : 1; // parity
        } value;
        
        unsigned long long raw;
    };

    // DCF Buffers and indicators
    unsigned long long bitmask;
    unsigned long long runningBuffer;

    // Pulse flanks
    unsigned long leadingEdgeTime;
    unsigned long trailingEdgeTime;

    //Private functions
    void InitBuffer(void);
    void finalizeBuffer(void);
    bool checkParity(unsigned long long buffer);
    void appendSignal(unsigned char signal);

    unsigned char lastYear;
    unsigned char lastMonth;
    unsigned char lastDay;
    unsigned long lastMinute;

public:
    
    // Public Functions
    DCF77();
    void int0handler();

    volatile DCFStatus rxStatus;
    volatile DCF77Buffer receivedTelegram;
    volatile unsigned long telegramTime;
};

extern DCF77 dcf;

#endif
