/* ------------------------------------------------------------
 * "THE BEERWARE LICENSE" (Revision 42):
 * <cvmanjoo@gmail.com> wrote this code. As long as you retain this
 * notice, you can do whatever you want with this stuff. If we
 * meet someday, and you think this stuff is worth it, you can
 * buy me a beer in return.
 * ------------------------------------------------------------
 * ------------------------------------------------------------
 * RTC.h - Library to set & get time from I2C RTCs for Arduino
 * Created by Manjunath CV. July 08, 2017, 02:18 AM
 * Released into the public domain.
 * -----------------------------------------------------------*/

#ifndef DS1307_H
#define DS1307_H

#if defined(ARDUINO_ARCH_RP2040)
#define UNIX_OFFSET 946684800
#endif

#define CLOCK_H24 0
#define CLOCK_H12 1

#define HOUR_AM 0
#define HOUR_PM 1
#define HOUR_24 2

#define SQW001Hz 10
#define SQW01kHz 11
#define SQW04kHz 14
#define SQW08kHz 18
#define SQW32kHz 32

#include <Arduino.h>
#include <time.h>
#include <Wire.h>

#define DS1307_ADDR 0x68

class DS1307
{
    public:
        bool begin();

        bool isRunning(void);
        void startClock(void);
        void stopClock();

        void setHourMode(uint8_t h_mode);
        uint8_t getHourMode();

        void setMeridiem(uint8_t meridiem);
        uint8_t getMeridiem();

        void setSeconds(uint8_t seconds);
        uint8_t getSeconds();
        
        void setMinutes(uint8_t minutes);
        uint8_t getMinutes();       

        void setHours(uint8_t hours);
        uint8_t getHours();       

        void setDay(uint8_t day);
        uint8_t getDay();

        uint8_t getWeek();
        void setWeek(uint8_t week);
        void updateWeek();

        void setMonth(uint8_t month);
        uint8_t getMonth();       

        uint8_t getYear();        
        void setYear(uint8_t year);

        void setAlarmSecond(uint8_t seconds);
        uint8_t getAlarmSecond();
        
        void setAlarmMinute(uint8_t minutes);
        uint8_t getAlarmMinute();       

        void setAlarmHour(uint8_t hours);
        uint8_t getAlarmHour();       

        void setAlarmFlags(uint8_t flags);
        uint8_t getAlarmFlags();       

        void setOutPin(uint8_t mode);
        bool isOutPinEnabled();
        bool isSqweEnabled();

        uint8_t bin2bcd (uint8_t val);
        uint8_t bcd2bin (uint8_t val);

private:
        uint8_t inline readRegister(uint8_t address);        
        void inline writeRegister(uint8_t address, uint8_t value);        
};

class NVRAM
{
    public:
        bool begin();
        uint8_t read(uint8_t address);
        void write(uint8_t address, uint8_t value);
        void read(uint8_t address, uint8_t *buf, uint8_t size);
        void write(uint8_t address, uint8_t *buf, uint8_t size);
        uint8_t length = 56;
    private:
};

#endif   /* RTC_H */
