

#include "rtc.h"
#include "dcf77.h"
#include "display.h"
#include "speaker.h"
#include "control.h"

void setup() {
    rtc_init();
    display_init();
    control_init();
    speaker_init();
    dcf_init();
}

unsigned long lastDCF = 0;

unsigned char hour, minute, second, day, month, year, alarm_hour, alarm_minute, alarm_second;
unsigned char last_tick;

void inline make_sound() {
    if (rtc.getAlarmFlags() && alarm_hour == hour && alarm_minute == minute && alarm_second == second) {
        alarm(30);
        return;
    }
    
    if (second == 0x00 && hour > 0x07 && hour < 0x20) {
        if (minute == 0x00) {
            if (option2() && option3()) {
                cuckoo(4, rtc.bcd2bin(hour));
                return; 
            }
            if (option2()) { 
                cuckoo2(rtc.bcd2bin(hour));
                return;
            }
            if (option3()) { 
                cuckoo1(4);
                return;
            }
        }
        if (minute == 0x15 && option3()) {
            cuckoo1(1);
            return;
        }
        if (minute == 0x30 && option3()) { 
            cuckoo1(2);
            return;
        } 
        if (minute == 0x45 && option3()) { 
            cuckoo1(3);
            return;
        } 
    }
}

void inline update_time() {
    if (digitalRead(RTC_CLOCK) != last_tick) {
        last_tick = ! last_tick;
        if (last_tick == LOW) {
            hour = rtc.getHours();
            minute = rtc.getMinutes();
            second = rtc.getSeconds();
            day = rtc.getDay();
            month = rtc.getMonth();
            year = rtc.getYear();
            alarm_hour = rtc.getAlarmHour();
            alarm_minute = rtc.getAlarmMinute();
            alarm_second = rtc.getAlarmSecond();

            make_sound();
        }
    }
}

void inline show_time() {
    PORT_HOUR = hour;
    PORT_MINUTE = minute;
    PORT_SECOND = second;
    if (last_tick)
        dots_off();
    else
        colons_on();
}

void inline show_date() {
    PORT_HOUR = (day & 0xF0) == 0x00 ? day | 0xF0 : day;
    PORT_MINUTE = (month & 0xF0) == 0x00 ? month | 0xF0 : month;
    PORT_SECOND = year;
    dots_on();
}

void inline show_alarm() {
    PORT_HOUR = alarm_hour;
    PORT_MINUTE = alarm_minute;
    PORT_SECOND = alarm_second;
    colons_on();
}

void inline show_alarm_hour() {
    if (last_tick == LOW) {
        PORT_HOUR = get_modified_value();
    } else {
        PORT_HOUR = 0xFF;
    }
    PORT_MINUTE = alarm_minute;
    PORT_SECOND = alarm_second;
    colons_on();
}

void inline show_alarm_minute() {
    PORT_HOUR = alarm_hour;
    if (last_tick == LOW) {
        PORT_MINUTE = get_modified_value();
    } else {
        PORT_MINUTE = 0xFF;
    }
    PORT_SECOND = alarm_second;
    colons_on();
}

void inline show_alarm_second() {
    PORT_HOUR = alarm_hour;
    PORT_MINUTE = alarm_minute;
    if (last_tick == LOW) {
        PORT_SECOND = get_modified_value();
    } else {
        PORT_SECOND = 0xFF;
    }
    colons_on();
}

void loop() {

    if (dcf.rxStatus == confirmed && (lastDCF != dcf.telegramTime)) {
        lastDCF = dcf.telegramTime;
        rtc.stopClock();
        rtc.setSeconds(0);
        rtc.setMinutes(dcf.receivedTelegram.value.Min);
        rtc.setHours(dcf.receivedTelegram.value.Hour);
        rtc.setDay(dcf.receivedTelegram.value.Day);
        rtc.setMonth(dcf.receivedTelegram.value.Month);
        rtc.setYear(dcf.receivedTelegram.value.Year);
        rtc.startClock();
    }

    update_time();

    unsigned char mode = get_mode();
    switch (mode) {
        case SHOW_TIME:
            show_time();
        break;
        case SHOW_DATE:
            show_date();
        break;
        case SHOW_ALARM:
            show_alarm();
        break;
        case SET_ALARM_HOUR:
            show_alarm_hour();
        break;
        case SET_ALARM_MINUTE:
            show_alarm_minute();
        break;
        case SET_ALARM_SECOND:
            show_alarm_second();
        break;
    }
}
